@@ -6,43 +6,28 @@
  * Copyright (C) 2004 Erik Andersen <andersen@codepoet.org>
  * Copyright (C) 2001 Matt Krai
  *
- * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
-
 #include "libbb.h"
 
-/* This function reads an entire line from a text file, up to a newline
- * or NUL byte, inclusive.  It returns a malloc'ed char * which
- * must be free'ed by the caller.  If end is NULL '\n' isn't considered
- * end of line.  If end isn't NULL, length of the chunk is stored in it.
- * If lineno is not NULL, *lineno is incremented for each line,
- * and also trailing '\' is recognized as line continuation.
- *
- * Returns NULL if EOF/error. */
-char* FAST_FUNC bb_get_chunk_with_continuation(FILE *file, int *end, int *lineno)
+char* FAST_FUNC bb_get_chunk_from_file(FILE *file, size_t *end)
 {
 	int ch;
-	int idx = 0;
+	size_t idx = 0;
 	char *linebuf = NULL;
-	int linebufsz = 0;
 
 	while ((ch = getc(file)) != EOF) {
 		/* grow the line buffer as necessary */
-		if (idx >= linebufsz) {
-			linebufsz += 256;
-			linebuf = xrealloc(linebuf, linebufsz);
+		if (!(idx & 0xff)) {
+			if (idx == ((size_t)-1) - 0xff)
+				bb_die_memory_exhausted();
+			linebuf = xrealloc(linebuf, idx + 0x100);
 		}
 		linebuf[idx++] = (char) ch;
-		if (!ch)
+		if (ch == '\0')
+			break;
+		if (end && ch == '\n')
 			break;
-		if (end && ch == '\n') {
-			if (lineno == NULL)
-				break;
-			(*lineno)++;
-			if (idx < 2 || linebuf[idx-2] != '\\')
-				break;
-			idx -= 2;
-		}
 	}
 	if (end)
 		*end = idx;
@@ -59,22 +44,15 @@ char* FAST_FUNC bb_get_chunk_with_continuation(FILE *file, int *end, int *lineno
 	return linebuf;
 }
 
-char* FAST_FUNC bb_get_chunk_from_file(FILE *file, int *end)
-{
-	return bb_get_chunk_with_continuation(file, end, NULL);
-}
-
 /* Get line, including trailing \n if any */
 char* FAST_FUNC xmalloc_fgets(FILE *file)
 {
-	int i;
-
-	return bb_get_chunk_from_file(file, &i);
+	return bb_get_chunk_from_file(file, NULL);
 }
 /* Get line.  Remove trailing \n */
 char* FAST_FUNC xmalloc_fgetline(FILE *file)
 {
-	int i;
+	size_t i;
 	char *c = bb_get_chunk_from_file(file, &i);
 
 	if (i && c[--i] == '\n')
